#!/usr/bin/env python3
"""
巨量引擎 Dify 兼容 MCP 服务器
专为 Dify 平台设计的 MCP 服务器
"""

from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse
from fastapi.middleware.cors import CORSMiddleware
import uvicorn
import json
import logging
from .server import create_asset_api, create_product_api, create_event_api

# 配置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

app = FastAPI(
    title="OceanEngine Dify MCP Server",
    description="Dify Compatible MCP Server for OceanEngine API operations",
    version="1.0.0"
)

# 添加 CORS 中间件
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# 工具定义
TOOLS = [
    {
        "name": "create_product",
        "description": "创建DPA商品",
        "parameters": {
            "type": "object",
            "properties": {
                "advertiser_id": {"type": "string", "description": "广告主ID"},
                "access_token": {"type": "string", "description": "访问令牌"},
                "platform_id": {"type": "string", "description": "商品库ID"},
                "product_name": {"type": "string", "description": "商品名称"},
                "image_url": {"type": "string", "description": "商品图片URL", "default": ""}
            },
            "required": ["advertiser_id", "access_token", "platform_id", "product_name"]
        }
    },
    {
        "name": "create_asset_and_event",
        "description": "创建资产并自动创建事件",
        "parameters": {
            "type": "object",
            "properties": {
                "advertiser_id": {"type": "string", "description": "广告主ID"},
                "access_token": {"type": "string", "description": "访问令牌"},
                "asset_name": {"type": "string", "description": "资产名称", "default": "dsok"},
                "asset_description": {"type": "string", "description": "资产描述", "default": ""},
                "event_id": {"type": "integer", "description": "事件ID", "default": 20}
            },
            "required": ["advertiser_id", "access_token"]
        }
    }
]


@app.get("/")
async def root():
    """根路径 - 返回服务器信息"""
    return {
        "name": "OceanEngine Dify MCP Server",
        "version": "1.0.0",
        "protocol": "Dify MCP",
        "transport": "HTTP",
        "compatible": "Dify MCP Service"
    }


@app.get("/tools")
async def get_tools():
    """获取工具列表 - Dify 格式"""
    return {
        "tools": TOOLS
    }


@app.post("/tools/{tool_name}")
async def call_tool(tool_name: str, request: Request):
    """调用工具 - Dify 格式"""
    try:
        # 解析请求体
        body = await request.json()
        logger.info(f"Tool call: {tool_name} with args: {body}")
        
        if tool_name == "create_product":
            result = create_product_api(
                body.get("advertiser_id"),
                body.get("access_token"),
                body.get("platform_id"),
                body.get("product_name"),
                body.get("image_url", "")
            )
            
            return {
                "result": json.dumps(result, ensure_ascii=False, indent=2)
            }
        
        elif tool_name == "create_asset_and_event":
            # 首先创建资产
            asset_result = create_asset_api(
                body.get("advertiser_id"),
                body.get("access_token"),
                body.get("asset_name", "dsok"),
                body.get("asset_description", "")
            )
            
            if asset_result.get('code') == 0:
                asset_id = asset_result.get('data', {}).get('asset_id')
                if asset_id:
                    # 然后创建事件
                    event_result = create_event_api(
                        body.get("advertiser_id"),
                        body.get("access_token"),
                        asset_id,
                        body.get("event_id", 20)
                    )
                    result = {
                        "asset_creation": asset_result,
                        "event_creation": event_result
                    }
                else:
                    result = {
                        "asset_creation": asset_result,
                        "event_creation": {"error": "资产创建未返回有效的asset_id"}
                    }
            else:
                result = {
                    "asset_creation": asset_result,
                    "event_creation": {"error": "资产创建失败"}
                }
            
            return {
                "result": json.dumps(result, ensure_ascii=False, indent=2)
            }
        
        else:
            return JSONResponse(
                status_code=404,
                content={"error": f"Unknown tool: {tool_name}"}
            )
    
    except Exception as e:
        logger.error(f"Tool call error: {str(e)}")
        return JSONResponse(
            status_code=500,
            content={"error": f"Internal error: {str(e)}"}
        )


def main():
    """启动 Dify MCP 服务器"""
    logger.info("Starting OceanEngine Dify MCP Server...")
    uvicorn.run(
        app, 
        host="0.0.0.0",  # 绑定到所有网络接口
        port=8000,
        log_level="info"
    )


if __name__ == "__main__":
    main()