#!/usr/bin/env python3
"""
巨量引擎 HTTP MCP 服务器 - 专为 Dify 设计
使用纯 HTTP 传输协议的 MCP 服务器
"""

from fastapi import FastAPI, HTTPException, Request
from fastapi.responses import JSONResponse
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel
from typing import Optional, Dict, Any, List
import uvicorn
import json
import logging
from .server import create_asset_api, create_product_api, create_event_api

# 配置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

app = FastAPI(
    title="OceanEngine HTTP MCP Server",
    description="HTTP MCP Server for OceanEngine API operations - Compatible with Dify",
    version="1.0.0"
)

# 添加 CORS 中间件
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


class MCPRequest(BaseModel):
    """MCP 请求模型"""
    jsonrpc: str = "2.0"
    id: Optional[str] = None
    method: str
    params: Optional[Dict[str, Any]] = None


class MCPResponse(BaseModel):
    """MCP 响应模型"""
    jsonrpc: str = "2.0"
    id: Optional[str] = None
    result: Optional[Dict[str, Any]] = None
    error: Optional[Dict[str, Any]] = None


# MCP 服务器信息
MCP_SERVER_INFO = {
    "name": "OceanEngine HTTP MCP Server",
    "version": "1.0.0",
    "protocolVersion": "2024-11-05",
    "capabilities": {
        "tools": {
            "listChanged": False
        }
    }
}

# 工具定义
TOOLS = [
    {
        "name": "create_product",
        "description": "创建DPA商品",
        "inputSchema": {
            "type": "object",
            "properties": {
                "advertiser_id": {"type": "string", "description": "广告主ID"},
                "access_token": {"type": "string", "description": "访问令牌"},
                "platform_id": {"type": "string", "description": "商品库ID"},
                "product_name": {"type": "string", "description": "商品名称"},
                "image_url": {"type": "string", "description": "商品图片URL", "default": ""}
            },
            "required": ["advertiser_id", "access_token", "platform_id", "product_name"]
        }
    },
    {
        "name": "create_asset_and_event",
        "description": "创建资产并自动创建事件",
        "inputSchema": {
            "type": "object",
            "properties": {
                "advertiser_id": {"type": "string", "description": "广告主ID"},
                "access_token": {"type": "string", "description": "访问令牌"},
                "asset_name": {"type": "string", "description": "资产名称", "default": "dsok"},
                "asset_description": {"type": "string", "description": "资产描述", "default": ""},
                "event_id": {"type": "integer", "description": "事件ID", "default": 20}
            },
            "required": ["advertiser_id", "access_token"]
        }
    }
]


@app.get("/")
async def root():
    """根路径 - 返回服务器信息"""
    return {
        "name": "OceanEngine HTTP MCP Server",
        "version": "1.0.0",
        "protocol": "HTTP MCP",
        "transport": "HTTP",
        "compatible": "Dify MCP Service",
        "endpoints": {
            "initialize": "POST /",
            "tools_list": "POST /",
            "tools_call": "POST /"
        }
    }


@app.post("/")
async def mcp_endpoint(request: Request):
    """HTTP MCP 主要端点 - 处理所有 MCP 请求"""
    # 直接从请求体中获取 JSON 数据，而不是使用 Pydantic 模型
    try:
        data = await request.json()
        logger.info(f"Received raw request: {data}")
    except Exception as e:
        logger.error(f"Failed to parse request JSON: {e}")
        return JSONResponse(
            status_code=400,
            content={"error": "Invalid JSON request"}
        )
    try:
        # 从请求数据中获取 MCP 相关字段
        method = data.get("method", "")
        request_id = data.get("id", "")
        params = data.get("params", {})
        
        logger.info(f"HTTP MCP Request: {method}")
        
        if method == "initialize":
            return {
                "jsonrpc": "2.0",
                "id": request_id,
                "result": {
                    "protocolVersion": "2024-11-05",
                    "capabilities": MCP_SERVER_INFO["capabilities"],
                    "serverInfo": {
                        "name": MCP_SERVER_INFO["name"],
                        "version": MCP_SERVER_INFO["version"]
                    }
                }
            }
        
        elif method == "tools/list":
            return {
                "jsonrpc": "2.0",
                "id": request_id,
                "result": {"tools": TOOLS}
            }
        
        elif method == "tools/call":
            tool_name = params.get("name")
            arguments = params.get("arguments", {})
            
            logger.info(f"Tool call: {tool_name} with args: {arguments}")
            
            if tool_name == "create_product":
                result = create_product_api(
                    arguments.get("advertiser_id"),
                    arguments.get("access_token"),
                    arguments.get("platform_id"),
                    arguments.get("product_name"),
                    arguments.get("image_url", "")
                )
                
                return {
                    "jsonrpc": "2.0",
                    "id": request_id,
                    "result": {
                        "content": [
                            {
                                "type": "text",
                                "text": json.dumps(result, ensure_ascii=False, indent=2)
                            }
                        ]
                    }
                }
            
            elif tool_name == "create_asset_and_event":
                # 首先创建资产
                asset_result = create_asset_api(
                    arguments.get("advertiser_id"),
                    arguments.get("access_token"),
                    arguments.get("asset_name", "dsok"),
                    arguments.get("asset_description", "")
                )
                
                if asset_result.get('code') == 0:
                    asset_id = asset_result.get('data', {}).get('asset_id')
                    if asset_id:
                        # 然后创建事件
                        event_result = create_event_api(
                            arguments.get("advertiser_id"),
                            arguments.get("access_token"),
                            asset_id,
                            arguments.get("event_id", 20)
                        )
                        result = {
                            "asset_creation": asset_result,
                            "event_creation": event_result
                        }
                    else:
                        result = {
                            "asset_creation": asset_result,
                            "event_creation": {"error": "资产创建未返回有效的asset_id"}
                        }
                else:
                    result = {
                        "asset_creation": asset_result,
                        "event_creation": {"error": "资产创建失败"}
                    }
                
                return {
                    "jsonrpc": "2.0",
                    "id": request_id,
                    "result": {
                        "content": [
                            {
                                "type": "text",
                                "text": json.dumps(result, ensure_ascii=False, indent=2)
                            }
                        ]
                    }
                }
            
            else:
                return {
                    "jsonrpc": "2.0",
                    "id": request_id,
                    "error": {
                        "code": -32601,
                        "message": f"Unknown tool: {tool_name}"
                    }
                }
        
        else:
            return {
                "jsonrpc": "2.0",
                "id": request_id,
                "error": {
                    "code": -32601,
                    "message": f"Method not found: {method}"
                }
            }
    
    except Exception as e:
        logger.error(f"HTTP MCP Error: {str(e)}")
        return {
            "jsonrpc": "2.0",
            "id": data.get("id", ""),
            "error": {
                "code": -32603,
                "message": f"Internal error: {str(e)}"
            }
        }


@app.options("/")
async def mcp_options():
    """处理 CORS 预检请求"""
    return JSONResponse(
        content={},
        headers={
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Methods": "GET, POST, OPTIONS",
            "Access-Control-Allow-Headers": "Content-Type, Authorization",
        }
    )


@app.get("/health")
async def health_check():
    """健康检查端点"""
    return {
        "status": "healthy",
        "server": MCP_SERVER_INFO["name"],
        "version": MCP_SERVER_INFO["version"],
        "transport": "HTTP",
        "protocol": "MCP over HTTP"
    }


@app.get("/capabilities")
async def get_capabilities():
    """获取服务器能力"""
    return {
        "protocolVersion": "2024-11-05",
        "capabilities": MCP_SERVER_INFO["capabilities"],
        "serverInfo": MCP_SERVER_INFO,
        "tools": TOOLS
    }


def main():
    """启动 HTTP MCP 服务器"""
    logger.info("Starting OceanEngine HTTP MCP Server for Dify...")
    uvicorn.run(
        app, 
        host="0.0.0.0",  # 绑定到所有网络接口
        port=8000,
        log_level="info"
    )


if __name__ == "__main__":
    main()