#!/usr/bin/env python3
"""
测试 advertiser_id 类型转换修复
"""

import requests
import json

def test_advertiser_id_conversion():
    """测试 advertiser_id 类型转换"""
    print("🧪 测试 advertiser_id 类型转换修复...")
    
    # 测试数据
    test_data = {
        "advertiser_id": "1834170090329099",  # 字符串格式
        "access_token": "c79fb626afceccede85e0b98e9ac772e808353a3",
        "asset_name": "测试资产",
        "asset_description": "测试描述"
    }
    
    try:
        # 测试 MCP 端点
        mcp_request = {
            "jsonrpc": "2.0",
            "id": "test-1",
            "method": "tools/call",
            "params": {
                "name": "create_asset_and_event",
                "arguments": test_data
            }
        }
        
        response = requests.post(
            "http://127.0.0.1:8000/mcp",
            json=mcp_request,
            headers={"Content-Type": "application/json"}
        )
        
        if response.status_code == 200:
            result = response.json()
            print("✅ MCP 端点响应成功")
            
            # 检查是否包含类型转换相关的错误
            result_text = str(result)
            if "Field must be set to integer" in result_text:
                print("❌ advertiser_id 类型转换失败 - 仍然出现整数类型错误")
                return False
            elif "Invalid advertiser_id" in result_text:
                print("❌ advertiser_id 格式无效")
                return False
            else:
                print("✅ advertiser_id 类型转换成功 - 没有出现类型错误")
                print(f"📄 响应内容: {json.dumps(result, indent=2, ensure_ascii=False)}")
                return True
        else:
            print(f"❌ HTTP 请求失败: {response.status_code}")
            return False
            
    except Exception as e:
        print(f"❌ 测试失败: {str(e)}")
        return False

def test_direct_api_call():
    """直接测试 API 函数"""
    print("\n🔧 直接测试 API 函数...")
    
    try:
        from oceanengine_mcp_server.server import create_asset_api
        
        # 测试字符串 advertiser_id
        result = create_asset_api("1834170090329099", "test_token", "测试资产")
        
        if "Invalid advertiser_id" in str(result):
            print("❌ 直接 API 调用失败 - advertiser_id 转换错误")
            return False
        else:
            print("✅ 直接 API 调用成功 - advertiser_id 正确转换为整数")
            print(f"📄 结果: {json.dumps(result, indent=2, ensure_ascii=False)}")
            return True
            
    except Exception as e:
        print(f"❌ 直接测试失败: {str(e)}")
        return False

def main():
    """主测试函数"""
    print("🚀 开始测试 advertiser_id 类型转换修复\n")
    
    tests = [
        ("直接 API 函数测试", test_direct_api_call),
        ("MCP 端点测试", test_advertiser_id_conversion),
    ]
    
    passed = 0
    total = len(tests)
    
    for test_name, test_func in tests:
        print(f"--- {test_name} ---")
        if test_func():
            passed += 1
        print()
    
    print(f"📊 测试结果: {passed}/{total} 通过")
    
    if passed == total:
        print("🎉 advertiser_id 类型转换修复成功！")
        print("✅ 现在可以正常使用字符串格式的 advertiser_id")
    else:
        print("⚠️  部分测试失败，需要进一步检查")

if __name__ == "__main__":
    main()