#!/usr/bin/env python3
"""
测试 MCP 服务器功能的脚本
"""

import json
import subprocess
import sys
import time
import threading


def test_mcp_server():
    """测试 MCP 服务器的基本功能"""
    print("🧪 测试 MCP 服务器...")
    
    # MCP 初始化消息
    init_message = {
        "jsonrpc": "2.0",
        "id": 1,
        "method": "initialize",
        "params": {
            "protocolVersion": "2024-11-05",
            "capabilities": {
                "tools": {}
            },
            "clientInfo": {
                "name": "test-client",
                "version": "1.0.0"
            }
        }
    }
    
    try:
        # 启动服务器进程
        process = subprocess.Popen(
            ['oceanengine-mcp-server'],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        
        # 发送初始化消息
        init_json = json.dumps(init_message) + '\n'
        process.stdin.write(init_json)
        process.stdin.flush()
        
        # 设置超时读取
        def read_with_timeout():
            try:
                return process.stdout.readline()
            except:
                return None
        
        # 等待响应
        response = None
        for _ in range(10):  # 等待最多1秒
            if process.poll() is not None:
                break
            try:
                line = process.stdout.readline()
                if line.strip():
                    response = line.strip()
                    break
            except:
                pass
            time.sleep(0.1)
        
        # 终止进程
        process.terminate()
        process.wait(timeout=2)
        
        if response:
            print(f"✅ 服务器响应: {response}")
            try:
                resp_data = json.loads(response)
                if 'result' in resp_data:
                    print("✅ MCP 服务器初始化成功")
                    return True
            except json.JSONDecodeError:
                pass
        
        print("⚠️  服务器启动但未收到预期响应")
        return False
        
    except Exception as e:
        print(f"❌ 测试失败: {e}")
        return False


def test_import():
    """测试模块导入"""
    print("📦 测试模块导入...")
    try:
        from oceanengine_mcp_server.server import create_product_api, create_asset_api
        print("✅ 模块导入成功")
        return True
    except ImportError as e:
        print(f"❌ 模块导入失败: {e}")
        return False


def test_api_functions():
    """测试 API 函数（使用无效参数，预期返回错误）"""
    print("🔧 测试 API 函数...")
    try:
        from oceanengine_mcp_server.server import create_product_api, create_asset_api
        
        # 测试创建商品 API（预期返回错误，因为参数无效）
        result1 = create_product_api("test", "test", "test", "test_product")
        if isinstance(result1, dict):
            print("✅ create_product_api 函数正常")
        
        # 测试创建资产 API（预期返回错误，因为参数无效）
        result2 = create_asset_api("test", "test", "test_asset")
        if isinstance(result2, dict):
            print("✅ create_asset_api 函数正常")
        
        return True
    except Exception as e:
        print(f"❌ API 函数测试失败: {e}")
        return False


def main():
    """主测试函数"""
    print("🚀 开始测试 oceanengine-mcp-server\n")
    
    tests = [
        ("模块导入测试", test_import),
        ("API 函数测试", test_api_functions),
        ("MCP 服务器测试", test_mcp_server),
    ]
    
    passed = 0
    total = len(tests)
    
    for test_name, test_func in tests:
        print(f"\n--- {test_name} ---")
        if test_func():
            passed += 1
        print()
    
    print(f"📊 测试结果: {passed}/{total} 通过")
    
    if passed >= 2:  # 至少模块导入和API函数测试通过
        print("🎉 基本功能测试通过！")
        print("\n📝 MCP 客户端配置:")
        print("对于本地测试（直接命令）:")
        print('{"command": "oceanengine-mcp-server"}')
        print("\n对于 uvx 安装后:")
        print('{"command": "uvx", "args": ["oceanengine-mcp-server"]}')
        print("\n⚠️  注意: MCP 服务器通过 stdio 通信，直接运行会等待输入，这是正常的")
    else:
        print("⚠️  部分测试失败，请检查问题")
        sys.exit(1)


if __name__ == "__main__":
    main()