#!/usr/bin/env python3
"""
巨量引擎 MCP 服务器

该服务器提供以下功能：
1. 创建第三方外部资产
2. 为资产创建事件
3. 创建DPA商品
4. 组合操作：创建资产并自动创建事件
"""

import json
import requests
from typing import Optional
from mcp.server.fastmcp import FastMCP


# 创建 MCP 服务器实例
mcp = FastMCP("OceanEngine MCP Server", 
              dependencies=["requests"])


# 预定义的 audience 模板
AUDIENCE_TEMPLATES = {
    "moban1": {
        "gender": "GENDER_FEMALE",
        "age": ["AGE_BETWEEN_18_23", "AGE_BETWEEN_24_30"],
        "city": ["340800", "340300"],
        "district": "REGION",
        "location_type": "HOME",
        "region_version": "2.3.2",
        "interest_action_mode": "UNLIMITED"
    },
    "moban2": {
        "gender": "GENDER_MALE",
        "age": ["AGE_BETWEEN_25_34", "AGE_BETWEEN_35_44"],
        "city": ["110100", "310100"],  # 北京、上海
        "district": "REGION",
        "location_type": "HOME",
        "region_version": "2.3.2",
        "interest_action_mode": "UNLIMITED"
    },
    "moban3": {
        "gender": "GENDER_UNLIMITED",
        "age": ["AGE_BETWEEN_18_23", "AGE_BETWEEN_24_30", "AGE_BETWEEN_31_40"],
        "city": ["440100", "440300"],  # 广州、深圳
        "district": "REGION",
        "location_type": "HOME",
        "region_version": "2.3.2",
        "interest_action_mode": "UNLIMITED"
    },
    "moban4": {
        "gender": "GENDER_FEMALE",
        "age": ["AGE_BETWEEN_18_23", "AGE_BETWEEN_24_30"],
        "city": ["320100", "330100"],  # 南京、杭州
        "district": "REGION",
        "location_type": "WORK",
        "region_version": "2.3.2",
        "interest_action_mode": "CUSTOM",
        "interest_categories": ["兴趣分类1", "兴趣分类2"]
    }
}


def get_audience_template(template_name: str) -> dict:
    """
    获取预定义的 audience 模板
    
    参数:
    - template_name: 模板名称 (moban1, moban2, moban3, moban4)
    
    返回:
    - audience 配置字典，如果模板不存在则返回 None
    """
    return AUDIENCE_TEMPLATES.get(template_name)


def get_aweme_auth_list_api(access_token: str, advertiser_id: str, page: int = 1, page_size: int = 10) -> dict:
    """
    调用巨量引擎API获取账户绑定的抖音号列表
    
    参数:
    - access_token: 访问令牌
    - advertiser_id: 广告主ID（字符串，会自动转换为整数）
    - page: 页码，默认为1
    - page_size: 每页数量，默认为10
    
    返回:
    - API响应结果字典
    """
    url = "https://api.oceanengine.com/open_api/2/tools/aweme_auth_list/"
    
    # 将 advertiser_id 转换为整数
    try:
        advertiser_id_int = int(advertiser_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid advertiser_id: {advertiser_id}. Must be a valid integer."}
    
    headers = {
        "Access-Token": access_token
    }
    
    params = {
        "advertiser_id": advertiser_id_int,
        "page": page,
        "page_size": page_size
    }
    
    try:
        response = requests.get(url, headers=headers, params=params)
        return response.json()
    except Exception as e:
        return {"error": str(e)}


def create_asset_api(advertiser_id: str, access_token: str, name: str = "dsok", description: str = "") -> dict:
    """
    调用巨量引擎API创建第三方外部资产
    
    参数:
    - advertiser_id: 广告主ID（字符串，会自动转换为整数）
    - access_token: 访问令牌
    - name: 资产名称
    - description: 资产描述
    
    返回:
    - API响应结果字典
    """
    url = "https://api.oceanengine.com/open_api/2/event_manager/assets/create/"
    
    # 将 advertiser_id 转换为整数
    try:
        advertiser_id_int = int(advertiser_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid advertiser_id: {advertiser_id}. Must be a valid integer."}
    
    payload = {
        "advertiser_id": advertiser_id_int,
        "asset_type": "THIRD_EXTERNAL",
        "third_part_asset": {
            "name": name,
            "description": description
        }
    }
    
    headers = {
        "Content-Type": "application/json",
        "Access-Token": access_token
    }
    
    try:
        response = requests.post(url, headers=headers, json=payload)
        return response.json()
    except Exception as e:
        return {"error": str(e)}


def create_event_api(advertiser_id: str, access_token: str, asset_id: str, event_id: int = 20) -> dict:
    """
    调用巨量引擎API为资产创建事件
    
    参数:
    - advertiser_id: 广告主ID（字符串，会自动转换为整数）
    - access_token: 访问令牌
    - asset_id: 资产ID
    - event_id: 事件ID，默认为20
    
    返回:
    - API响应结果字典
    """
    url = "https://api.oceanengine.com/open_api/2/event_manager/events/create/"
    
    # 将 advertiser_id 转换为整数
    try:
        advertiser_id_int = int(advertiser_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid advertiser_id: {advertiser_id}. Must be a valid integer."}
    
    payload = {
        "advertiser_id": advertiser_id_int,
        "asset_id": asset_id,
        "event_id": event_id,
        "track_types": ["EXTERNAL_API"]
    }
    
    headers = {
        "Content-Type": "application/json",
        "Access-Token": access_token
    }
    
    try:
        response = requests.post(url, headers=headers, json=payload)
        return response.json()
    except Exception as e:
        return {"error": str(e)}


def create_product_api(advertiser_id: str, access_token: str, platform_id: str, 
                      product_name: str, image_url: str = "") -> dict:
    """
    调用巨量引擎API创建DPA商品
    
    参数:
    - advertiser_id: 广告主ID（字符串，会自动转换为整数）
    - access_token: 访问令牌
    - platform_id: 商品库ID
    - product_name: 商品名称
    - image_url: 商品图片URL
    
    返回:
    - API响应结果字典
    """
    url = "https://api.oceanengine.com/open_api/2/dpa/product/create/"
    
    # 将 advertiser_id 转换为整数
    try:
        advertiser_id_int = int(advertiser_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid advertiser_id: {advertiser_id}. Must be a valid integer."}
    
    # 将 platform_id 转换为整数
    try:
        platform_id_int = int(platform_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid platform_id: {platform_id}. Must be a valid integer."}
    
    payload = {
        "advertiser_id": advertiser_id_int,
        "platform_id": platform_id_int,
        "product_info": {
            "name": product_name,
            "title": "",
            "titles": [""],
            "outer_id": "",
            "description": "",
            "image_url": image_url,
            "profession": {
                "platform_v2": "淘宝"
            }
        }
    }
    
    headers = {
        "Content-Type": "application/json",
        "Access-Token": access_token
    }
    
    try:
        response = requests.post(url, headers=headers, json=payload)
        return response.json()
    except Exception as e:
        return {"error": str(e)}


def create_project_api(
    access_token: str,
    advertiser_id: str,
    name: str,
    product_platform_id: str,  # 商品库ID
    product_id: str,          # 商品ID
    asset_id: str,            # 资产ID
    budget: int,
    cpa_bid: int,
    audience: dict
) -> dict:
    """
    调用巨量引擎API创建项目（PROCEDURAL投放）

    参数:
    - access_token: 访问令牌
    - advertiser_id: 广告主ID
    - name: 项目名称
    - product_platform_id: 商品库ID
    - product_id: 商品ID
    - asset_id: 资产ID
    - budget: 日预算金额
    - cpa_bid: CPA出价金额
    - audience: audience定向配置（dict，需包含所有定向信息）

    返回:
    - API响应结果字典
    """
    # 转换ID为整数
    try:
        advertiser_id_int = int(advertiser_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid advertiser_id: {advertiser_id}. Must be a valid integer."}
    
    try:
        product_platform_id_int = int(product_platform_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid product_platform_id: {product_platform_id}. Must be a valid integer."}
    
    try:
        product_id_int = int(product_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid product_id: {product_id}. Must be a valid integer."}
    
    try:
        asset_id_int = int(asset_id)
    except (ValueError, TypeError):
        return {"error": f"Invalid asset_id: {asset_id}. Must be a valid integer."}
    
    # 处理 audience 参数，确保 city 是字符串数组
    processed_audience = audience.copy()
    if 'city' in processed_audience and isinstance(processed_audience['city'], list):
        processed_audience['city'] = [str(city) for city in processed_audience['city']]
    
    # 确保其他可能需要字符串的字段也被正确处理
    for key in processed_audience:
        if isinstance(processed_audience[key], list) and processed_audience[key] and isinstance(processed_audience[key][0], int):
            processed_audience[key] = [str(item) for item in processed_audience[key]]

    url = 'https://api.oceanengine.com/open_api/v3.0/project/create/'
    headers = {
        'Content-Type': 'application/json',
        'Access-Token': access_token
    }
    body = {
        'advertiser_id': advertiser_id_int,
        'name': name,
        'landing_type': 'SHOP',
        'marketing_goal': 'VIDEO_AND_IMAGE',
        'ad_type': 'ALL',
        'asset_type': 'THIRDPARTY',
        'related_product': {
            'product_setting': 'SINGLE',
            'product_platform_id': product_platform_id_int,
            'product_id': str(product_id_int)
        },
        'optimize_goal': {
            'asset_ids': [str(asset_id_int)],
            'external_action': 'AD_CONVERT_TYPE_APP_ORDER'
        },
        'delivery_range': {
            'inventory_catalog': 'UNIVERSAL_SMART'
        },
        'delivery_setting': {
            'schedule_type': 'SCHEDULE_FROM_NOW',
            'pricing': 'PRICING_OCPM',
            'bid_type': 'CUSTOM',
            'budget_mode': 'BUDGET_MODE_DAY',
            'budget': int(budget),
            'cpa_bid': int(cpa_bid)
        },
        'audience': processed_audience,
        'delivery_mode': 'PROCEDURAL'
    }
    
    # 添加详细日志记录
    debug_info = {
        'request_url': url,
        'request_headers': {k: v for k, v in headers.items() if k.lower() != 'access-token'},
        'request_body': body
    }
    
    try:
        response = requests.post(url, headers=headers, json=body)
        response_json = response.json()
        
        # 添加响应信息到调试信息
        debug_info['response_status'] = response.status_code
        debug_info['response_body'] = response_json
        
        # 如果响应中包含错误信息，添加到返回结果中
        if response.status_code != 200 or response_json.get('code') != 0:
            response_json['debug_info'] = debug_info
        
        return response_json
    except Exception as e:
        error_result = {'error': str(e), 'debug_info': debug_info}
        return error_result


@mcp.tool()
def create_product(advertiser_id: str, access_token: str, platform_id: str, 
                  product_name: str, image_url: str = "") -> str:
    """
    创建DPA商品
    
    参数:
    - advertiser_id: 广告主ID（必填）
    - access_token: 访问令牌（必填）
    - platform_id: 商品平台ID（必填）
    - product_name: 商品名称（必填）
    - image_url: 商品图片URL，默认为空
    
    返回:
    - 创建结果的JSON字符串
    """
    result = create_product_api(advertiser_id, access_token, platform_id, product_name, image_url)
    return json.dumps(result, indent=2, ensure_ascii=False)


@mcp.tool()
def create_asset_and_event(advertiser_id: str, access_token: str, asset_name: str = "dsok", 
                          asset_description: str = "", event_id: int = 20) -> str:
    """
    创建资产并自动为其创建事件（组合操作）
    
    参数:
    - advertiser_id: 广告主ID（必填）
    - access_token: 访问令牌（必填）
    - asset_name: 资产名称，默认为"dsok"
    - asset_description: 资产描述，默认为空
    - event_id: 事件ID，默认为20
    
    返回:
    - 包含资产创建和事件创建结果的JSON字符串
    """
    # 首先创建资产
    asset_result = create_asset_api(advertiser_id, access_token, asset_name, asset_description)
    
    if asset_result.get('code') == 0:
        asset_id = asset_result.get('data', {}).get('asset_id')
        if asset_id:
            # 然后创建事件
            event_result = create_event_api(advertiser_id, access_token, asset_id, event_id)
            combined_result = {
                "asset_creation": asset_result,
                "event_creation": event_result
            }
        else:
            combined_result = {
                "asset_creation": asset_result,
                "event_creation": {"error": "资产创建未返回有效的asset_id"}
            }
    else:
        combined_result = {
            "asset_creation": asset_result,
            "event_creation": {"error": "资产创建失败"}
        }
    
    return json.dumps(combined_result, indent=2, ensure_ascii=False)


@mcp.tool()
def get_aweme_auth_list(access_token: str, advertiser_id: str, page: int = 1, page_size: int = 10) -> str:
    """
    获取账户绑定的抖音号列表
    
    参数:
    - access_token: 访问令牌（必填）
    - advertiser_id: 广告主ID（必填）
    - page: 页码，默认为1
    - page_size: 每页数量，默认为10
    
    返回:
    - 抖音号授权列表的JSON字符串，包含aweme_id等信息
    """
    result = get_aweme_auth_list_api(access_token, advertiser_id, page, page_size)
    return json.dumps(result, indent=2, ensure_ascii=False)


@mcp.tool()
def create_project(
    access_token: str,
    advertiser_id: str,
    name: str,
    product_platform_id: str,
    product_id: str,
    asset_id: str,
    budget: int,
    cpa_bid: int,
    audience_template: str
) -> str:
    """
    创建巨量引擎项目（PROCEDURAL投放）

    参数:
    - access_token: 访问令牌（必填）
    - advertiser_id: 广告主ID（必填）
    - name: 项目名称（必填）
    - product_platform_id: 商品库ID（必填）
    - product_id: 商品ID（必填）
    - asset_id: 资产ID（必填）
    - budget: 日预算金额（必填）
    - cpa_bid: CPA出价金额（必填）
    - audience_template: audience模板名称（必填，可选值：moban1, moban2, moban3, moban4）

    返回:
    - 创建结果的JSON字符串
    """
    try:
        # 获取预定义的 audience 模板
        audience_dict = get_audience_template(audience_template)
        
        if audience_dict is None:
            available_templates = list(AUDIENCE_TEMPLATES.keys())
            error_result = {
                "error": f"Invalid audience template: {audience_template}. Available templates: {available_templates}"
            }
            return json.dumps(error_result, indent=2, ensure_ascii=False)
        
        result = create_project_api(
            access_token=access_token,
            advertiser_id=advertiser_id,
            name=name,
            product_platform_id=product_platform_id,
            product_id=product_id,
            asset_id=asset_id,
            budget=budget,
            cpa_bid=cpa_bid,
            audience=audience_dict
        )
        return json.dumps(result, indent=2, ensure_ascii=False)
    except Exception as e:
        error_result = {"error": f"Unexpected error: {str(e)}"}
        return json.dumps(error_result, indent=2, ensure_ascii=False)


@mcp.prompt()
def oceanengine_usage_guide() -> str:
    """巨量引擎MCP服务器使用指南"""
    return """
# 巨量引擎 MCP 服务器使用指南

## 功能说明
这个MCP服务器提供巨量引擎API的常用操作，包括资产管理、事件管理、商品管理和项目创建。

## 可用工具

### 1. create_product - 创建商品
创建DPA商品
- advertiser_id: 广告主ID（必填）
- access_token: 访问令牌（必填）
- platform_id: 商品平台ID（必填）
- product_name: 商品名称（必填）
- image_url: 商品图片URL（可选）

### 2. create_asset_and_event - 组合操作
创建资产并自动创建事件（推荐使用）
- advertiser_id: 广告主ID（必填）
- access_token: 访问令牌（必填）
- asset_name: 资产名称（可选，默认"dsok"）
- asset_description: 资产描述（可选，默认为空）
- event_id: 事件ID（可选，默认20）

### 3. get_aweme_auth_list - 获取抖音号授权列表
获取账户绑定的抖音号列表，包含aweme_id等信息
- access_token: 访问令牌（必填）
- advertiser_id: 广告主ID（必填）
- page: 页码（可选，默认为1）
- page_size: 每页数量（可选，默认为10）

### 4. create_project - 创建项目（PROCEDURAL投放）
创建巨量引擎项目，支持PROCEDURAL投放模式，使用预定义的audience模板。
- access_token: 访问令牌（必填）
- advertiser_id: 广告主ID（必填）
- name: 项目名称（必填）
- product_platform_id: 商品库ID（必填）
- product_id: 商品ID（必填）
- asset_id: 资产ID（必填）
- budget: 日预算金额（必填）
- cpa_bid: CPA出价金额（必填）
- audience_template: audience模板名称（必填，可选值：moban1, moban2, moban3, moban4）

## Audience 模板说明

### moban1 - 年轻女性用户
- 性别: 女性
- 年龄: 18-23岁, 24-30岁
- 城市: 芜湖、蚌埠
- 位置类型: 居住地

### moban2 - 中年男性用户
- 性别: 男性
- 年龄: 25-34岁, 35-44岁
- 城市: 北京、上海
- 位置类型: 居住地

### moban3 - 不限性别广泛用户
- 性别: 不限
- 年龄: 18-23岁, 24-30岁, 31-40岁
- 城市: 广州、深圳
- 位置类型: 居住地

### moban4 - 职场女性用户
- 性别: 女性
- 年龄: 18-23岁, 24-30岁
- 城市: 南京、杭州
- 位置类型: 工作地
- 兴趣定向: 自定义

## 使用示例

### 创建商品
```
create_product(
    advertiser_id="1234567890",
    access_token="your_access_token",
    platform_id="platform_123",
    product_name="我的商品",
    image_url="https://example.com/image.jpg"
)
```

### 创建资产并自动创建事件
```
create_asset_and_event(
    advertiser_id="1234567890",
    access_token="your_access_token",
    asset_name="我的资产",
    asset_description="资产描述",
    event_id=20
)
```

### 创建项目（使用模板）
```
create_project(
    access_token="your_access_token",
    advertiser_id="1234567890",
    name="测试项目",
    product_platform_id="商品库ID",
    product_id="商品ID",
    asset_id="资产ID",
    budget=1000,
    cpa_bid=50,
    audience_template="moban1"  # 使用预定义模板
)
```

## 注意事项
- asset_id 是资产ID（如第三方外部资产的ID）
- product_platform_id 是商品库ID
- product_id 是商品ID
- 现在使用 audience_template 参数，不再需要手动构建复杂的 audience 对象
- 可用的模板：moban1, moban2, moban3, moban4
- 如果需要新的定向配置，可以联系开发者添加新模板
"""


def main():
    """启动MCP服务器"""
    mcp.run()


if __name__ == "__main__":
    main()