#!/usr/bin/env python3
"""
巨量引擎 MCP 服务器测试客户端

用于测试 MCP 服务器的各种功能
"""

import json
import asyncio
from mcp import ClientSession, StdioServerParameters
from mcp.client.stdio import stdio_client


async def test_create_project():
    """测试创建项目功能"""
    
    # 测试参数
    test_params = {
        "access_token": "fdb52ca5276eaf832d756a305cd5ee1df93ec981",
        "advertiser_id": "1834170090329099",
        "name": "测试项目",
        "product_platform_id": "6383139949659930",
        "product_id": "1752731593405736728",
        "asset_id": "1837874387273299",
        "budget": 300,
        "cpa_bid": 3,
        "audience": json.dumps({
            "gender": "GENDER_FEMALE",
            "age": ["AGE_BETWEEN_18_23", "AGE_BETWEEN_24_30"],
            "city": ["340800", "340300"],  # 注意这里是字符串数组
            "district": "REGION",
            "location_type": "HOME",
            "region_version": "2.3.2",
            "interest_action_mode": "UNLIMITED"
        })
    }
    
    # 启动 MCP 服务器
    server_params = StdioServerParameters(
        command="python",
        args=["-m", "oceanengine_mcp_server.server"]
    )
    
    async with stdio_client(server_params) as (read, write):
        async with ClientSession(read, write) as session:
            # 初始化
            await session.initialize()
            
            # 列出可用工具
            tools = await session.list_tools()
            print("可用工具:")
            for tool in tools.tools:
                print(f"  - {tool.name}: {tool.description}")
            print()
            
            # 调用 create_project 工具
            print("调用 create_project 工具...")
            print(f"参数: {json.dumps(test_params, indent=2, ensure_ascii=False)}")
            print()
            
            try:
                result = await session.call_tool("create_project", test_params)
                print("调用结果:")
                print(json.dumps(result.content[0].text, indent=2, ensure_ascii=False))
            except Exception as e:
                print(f"调用失败: {e}")


async def test_create_asset_and_event():
    """测试创建资产和事件功能"""
    
    test_params = {
        "advertiser_id": "1834170090329099",
        "access_token": "fdb52ca5276eaf832d756a305cd5ee1df93ec981",
        "asset_name": "测试资产",
        "asset_description": "这是一个测试资产",
        "event_id": 20
    }
    
    server_params = StdioServerParameters(
        command="python",
        args=["-m", "oceanengine_mcp_server.server"]
    )
    
    async with stdio_client(server_params) as (read, write):
        async with ClientSession(read, write) as session:
            await session.initialize()
            
            print("调用 create_asset_and_event 工具...")
            print(f"参数: {json.dumps(test_params, indent=2, ensure_ascii=False)}")
            print()
            
            try:
                result = await session.call_tool("create_asset_and_event", test_params)
                print("调用结果:")
                print(json.dumps(result.content[0].text, indent=2, ensure_ascii=False))
            except Exception as e:
                print(f"调用失败: {e}")


async def test_create_product():
    """测试创建商品功能"""
    
    test_params = {
        "advertiser_id": "1834170090329099",
        "access_token": "fdb52ca5276eaf832d756a305cd5ee1df93ec981",
        "platform_id": "6383139949659930",
        "product_name": "测试商品",
        "image_url": "https://example.com/test-image.jpg"
    }
    
    server_params = StdioServerParameters(
        command="python",
        args=["-m", "oceanengine_mcp_server.server"]
    )
    
    async with stdio_client(server_params) as (read, write):
        async with ClientSession(read, write) as session:
            await session.initialize()
            
            print("调用 create_product 工具...")
            print(f"参数: {json.dumps(test_params, indent=2, ensure_ascii=False)}")
            print()
            
            try:
                result = await session.call_tool("create_product", test_params)
                print("调用结果:")
                print(json.dumps(result.content[0].text, indent=2, ensure_ascii=False))
            except Exception as e:
                print(f"调用失败: {e}")


def main():
    """主函数"""
    print("=== 巨量引擎 MCP 服务器测试客户端 ===\n")
    
    while True:
        print("请选择要测试的功能:")
        print("1. 测试创建项目 (create_project)")
        print("2. 测试创建资产和事件 (create_asset_and_event)")
        print("3. 测试创建商品 (create_product)")
        print("4. 退出")
        
        choice = input("\n请输入选择 (1-4): ").strip()
        
        if choice == "1":
            print("\n=== 测试创建项目 ===")
            asyncio.run(test_create_project())
        elif choice == "2":
            print("\n=== 测试创建资产和事件 ===")
            asyncio.run(test_create_asset_and_event())
        elif choice == "3":
            print("\n=== 测试创建商品 ===")
            asyncio.run(test_create_product())
        elif choice == "4":
            print("退出测试客户端")
            break
        else:
            print("无效选择，请重新输入")
        
        print("\n" + "="*50 + "\n")


if __name__ == "__main__":
    main()